/* compile.h
	P.Chongstitvatana

	6 Jan 2004   project som-c
	7 Nov 2005	 project som-v16
	30 Dec 2005	 som-v17
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "icname.h"
#include "token.h"

//#define DEBUG
//#define VERBOSE

#define PRIVATE			static

#define	TABLESIZE		499		// should be prime
#define LOCALSIZE       64		// is LVMAX
#define NAMELEN			32
#define CASESIZE		100
#define MAXYSTK			1000
#define MAXLBUF			1000

/* memory map
  M[ system | --> DS   | --> CS | cell ]
            ^          ^        ^      ^
           MAXSYS    MAXDS    MAXCS   MAXMEM

  SS[MAXSS]

   system area  1 .. MAXSYS-1
   1..63 local  64..255 global
   256..399 som-string area (143 words)
*/
#define MAXMEM			500000	// total memory
#define MAXCS			200000	// code segment
#define MAXDS           100000	// data segment
#define MAXSYS			1000	// interface C-som
#define MAXSS			5000	// stack size
#define MAXSTR			256		// max tokstring length
#define MAXGVAR			255		// max global
#define LVMAX			64		// max local
#define SSTRING			256		// som-string area

#define	tkIDEN		    14
#define	tkNUMBER        15
#define	tkSTRING        16
#define	tkEOF           17
#define	tkERROR         18

// type of identifier in symbol table
#define tyNEW			10
#define tyFUNC			6
#define tyLOCAL			7
#define tyGVAR			8
#define tyENUM			9
#define tyMAC			11	// macro
#define tyFULL			12	// full macro

// denote gvar arg for DS relocation
#define VSCALAR			0
#define VARRAY			1
#define VSTRING			2

#define NL_CHAR			10
#define EOF_CHAR		255

#define MARK			1	// marker in ystack stmt.c
#define BMARK			2	// break marker
#define NIL				0

// atom in parse tree
#define SP				0   // special
#define OPER			1   // operators
#define NUM				2   // number
#define GNAME			3	// global name
#define LNAME			4	// local name
#define	CNAME			5	// class name
#define STRING			6   // string
#define ADS				7	// pointer

#define SYSARRAY		10	// syscall array
#define RETVAL			64	// global for return value
#define OBJ_SOM17		5678917	// magic cookie

// opcode format
#define fA		1
#define fB		2
#define fC		3
#define fD		4
#define fE		5
#define fF		6

// for interp.c

#define	  FALSE		0
#define   FUDGE     16   // scratch area + 3  to chk stkovf

#define   MAXFILE	20
#define	  FREAD		0
#define	  FWRITE	1
#define   FRDWR		2

// -------

#define  eqs(a,b)  (strcmp(a,b) == 0)
#define  nl()		printf("\n")

typedef unsigned char uchar;

typedef struct {
  char name[NAMELEN];
  int  type;
  int  ref;			// point to code/parse-tree
  int  arg;			// arity
  int  lv;			// total no. of var
} sym_entry;

typedef struct{
	char pair;
	char free;
} ventry;

extern ventry vrec[];
extern  char tokstring[];	/* current token string */
extern  int tok;			/* current token */
extern  uchar CH;			/* current char */
extern  int *CS;			/* code segment */
extern  int M[];			/* data, code, cell */
extern  int DS;				/* free data pointer */
extern  int exsym[], nxsym;	// export symbol, in symtab.c
extern  int vmin, vmax;
extern  int CP;				// code pointer
extern  int line;			// current line in source

/* function prototypes */

void getC(void);				/* lex.c */
int isLetterOrDigit(uchar c);
void lex(void);
void readinfile(char *fname);
void startlex(void);
void prToken(int tok);
void prtok(int tok);
void reader(void);
void initCharType(void);
int atoi2(char *s);

void seterror( char *mess );	// som.c
void warning( char *s, char *mess);
void loadfile( char *fn );

int search(char *name);			// symtab.c
int enterLocal(char *name);
int getRef(int index);			// access fn to symtab
void setRef(int index, int ref);
int getArg(int idx);
void setArg(int idx, int arg);
int getLv(int idx);
void setLv(int idx, int lv);
int getType(int idx);
void setType(int index, int type);
char *getName(int nm);
void dumpSymTab(FILE *fo);
char *findName(int type, int ref);
int getExportSym(int *list);
void readSymTab(FILE *fi);

void outa( int code, int arg);	/* icode.c */
void outa2( int code, int a1, int a2);
void outa3( int code, int a1, int a2, int a3);
void showCode(FILE *fo);
void decode(int ip, int *code, int *data);
void putcode(int ip, int code, int data);
void clean(int idx);
void outCode(FILE *fo);
void outData(FILE *fo);
void readCode(FILE *fi);
void readData(FILE *fi);
void patchCallf(int a, int b);
void reLoc(int disp);
void patch( int ads, int v );
void patcha1( int ads, int v );
void patcha2( int ads, int v);
void patchd( int ads, int v );
void rename3(int op, int a, int base, int *n);
void unpack(int a, int *a1, int *a2, int *a3);
void prCode(FILE *fo, int ip);
void showCode2(int ip);
//void chkretval(int ip, int d);
void reName( int ref);
void outM(FILE *fo, int *mem, int start, int end);
int readM(FILE *fi, int *mem);

//int ex();						/* parse.c */
int ex0(void);
int mod(void);
int top(void);
int terms(void);

void expect( int tktype, char *mess );	/* stmt.c */
void commit( int status );
int exas(void);
int ypop(void);
void ypush(int item);
int ytos(void);
int getysp(void);
void setfname(int ty);
void setarity(void);
void dofun(int op);
void doblock(void);
void doif(void);
void dowhile(void);
void dofor(void);
void dosys(void);
void dobop(int bop);
void donum(int value);
void dovec(void);
void douop(int uop);
void docase(void);
void dolocal(void);
void doenum(void);
void doiden(int idx);
void dolabel(void);
void dostring(char *s);
int newdata(void);
void dostatic(int a);
void akeepStr(char *s);
void akeepIden(char *s);
void strpack(int s1, char *s2);		// string func
void strunpack(char *s1, int s2);
void prstr(int i);

void eval(int a);				/* interp.c */
int xalloc(int a);
void initfile(void);

void initcell(void);			/* list.c */
int newatom(int type, int value);
int isatom(int a);
//int islist(int a);
int car(int l);
int cdr(int l);
int cons(int a, int l);
int list(int a);
void prlist(int l);
int append(int l, int x);
//int countcell(void);
void setcar(int a, int val);
void setcdr(int a, int val);

int genex(int e);				/* gencode.c */
void runimm(int e);
void genfinal(void);
int genex2(int e);
void freev(int v);
int genex3(int e);

int domacro(int idx, int el);	// macro.c
